export var source = {
    name: "Twitter",
    auth: {
        oauth2: {
            type: "oauth2",
            token_url: "/oauth2/token",
            fields: [
                {
                    key: "consumerKey",
                    type: "username",
                },
                {
                    key: "consumerSecret",
                    type: "password",
                },
            ],
        },
    },
    apis: {
        json: {
            name: "JSON",
            auth: ["oauth2"],
            maxLimit: 200,
            search: {
                url: function (query, opts, previous) {
                    try {
                        var pageUrl = Grabber.pageUrl(query.page, previous, 1, "", "&since_id={max}", "&max_id={min-1}");
                        return "/1.1/statuses/user_timeline.json?include_rts=true&screen_name=" + encodeURIComponent(query.search) + pageUrl;
                    }
                    catch (e) {
                        return { error: e.message };
                    }
                },
                parse: function (src) {
                    var data = JSON.parse(src);
                    var images = [];
                    for (var i in data) {
                        var sc = data[i];
                        var d = {};
                        if (!("extended_entities" in sc)) {
                            continue;
                        }
                        var entities = sc["extended_entities"];
                        if (!("media" in entities)) {
                            continue;
                        }
                        var medias = entities["media"];
                        if (!medias || medias.length === 0) {
                            continue;
                        }
                        var media = medias[0];
                        var sizes = media["sizes"];
                        d["id"] = sc["id_str"];
                        d["created_at"] = sc["created_at"];
                        d["preview_url"] = media["media_url_https"] + ":thumb";
                        if ("medium" in sizes) {
                            d["sample_url"] = media["media_url_https"] + ":medium";
                        }
                        var size = sizes["large"];
                        d["width"] = size["w"];
                        d["height"] = size["h"];
                        if (media.contains("video_info")) {
                            var maxBitrate = -1;
                            for (var _i = 0, _a = media["video_info"]["variants"]; _i < _a.length; _i++) {
                                var variantInfo = _a[_i];
                                var bitrate = variantInfo["bitrate"];
                                if (bitrate > maxBitrate) {
                                    maxBitrate = bitrate;
                                    d["file_url"] = variantInfo["url"];
                                }
                            }
                        }
                        else {
                            d["file_url"] = media["media_url_https"] + ":large";
                        }
                        images.push(d);
                    }
                    return { images: images };
                },
            },
        },
    },
};
